/************************************************************************/
/*																		*/
/*	Dclock.c	--	Interface Declarations for PIC32 Clock Generation	*/
/*																		*/
/************************************************************************/
/*	Author:		Oliver Jones											*/
/*	Copyright 2011, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*																		*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	07/19/2011(OliverJ): created										*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include <p32xxxx.h>
#include "Dclock.h"

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*				Interrupt Declarations							*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */
/***	unsigned int DclockGetSysClock()
**
**	Parameters:
**		none
**
**	Return Value:
**		Returns unsigned int representing clock frq
**
**	Errors:
**		none
**
**	Description:
**		Checks through the OSCCON Register and using a known
**		frequncy of the board crystal returns a clock frequency
**
*/

unsigned int DclockGetSysClock()
{
	unsigned int sysClock = 0;
	unsigned int plloDiv = 0;
	unsigned int frcDiv = 0;
	unsigned int pllMult = 0;

	sysClock = crystalFrq;

	plloDiv = OSCCON & bmPLLODIV;

	frcDiv = OSCCON & bmFRCDIV;

	pllMult = OSCCON & bmPLLMULT;

	switch(plloDiv) {
		case pllDiv256:
			sysClock = sysClock / 256;
			break;
		
		case pllDiv64:
			sysClock = sysClock / 64;
			break;

		case pllDiv32:
			sysClock = sysClock / 32;
			break;

		case pllDiv16:
			sysClock = sysClock / 16;
			break;

		case pllDiv8:
			sysClock = sysClock / 8;
			break;

		case pllDiv4:
			sysClock = sysClock / 4;
			break;
		
		case pllDiv2:
			sysClock = sysClock / 2;
			break;

		case pllDiv1:
			sysClock = sysClock / 1;
	}

	switch(frcDiv) {
		case frcDiv256:
			sysClock = sysClock / 256;
			break;
		
		case frcDiv64:
			sysClock = sysClock / 64;
			break;

		case frcDiv32:
			sysClock = sysClock / 32;
			break;

		case frcDiv16:
			sysClock = sysClock / 16;
			break;

		case frcDiv8:
			sysClock = sysClock / 8;
			break;

		case frcDiv4:
			sysClock = sysClock / 4;
			break;
		
		case frcDiv2:
			sysClock = sysClock / 2;
			break;

		case frcDiv1:
			sysClock = sysClock / 1;
	}

	switch(pllMult) {
		case pllMult24:
			sysClock = sysClock * 24;
			break;
		
		case pllMult21:
			sysClock = sysClock * 21;
			break;

		case pllMult20:
			sysClock = sysClock * 20;
			break;

		case pllMult19:
			sysClock = sysClock * 19;
			break;

		case pllMult18:
			sysClock = sysClock * 18;
			break;

		case pllMult17:
			sysClock = sysClock * 17;
			break;

		case pllMult16:
			sysClock = sysClock * 16;
			break;
		
		case pllMult15:
			sysClock = sysClock * 15;
			break;
	}

	return sysClock;
}

/* ------------------------------------------------------------ */
/***	unsigned int DclockGetPbClock()
**
**	Parameters:
**		none
**
**	Return Value:
**		Returns unsigned int representing pbclock frq
**
**	Errors:
**		none
**
**	Description:
**		Retrieves the system clock frequency using DclockGetSysClock()
**		and then uses OSCCON to calculate the peripheral clock
**
*/


unsigned int DclockGetPbClock()
{
	unsigned int pbClock;
	unsigned int sysClock;
	unsigned int pbDiv = 0;

	sysClock = DclockGetSysClock();

	pbDiv = OSCCON & bmPBDIV;

	switch(pbDiv) {
		case pbDiv8:
			pbClock = sysClock / 8;
			break;
		
		case pbDiv4:
			pbClock = sysClock / 4;
			break;

		case pbDiv2:
			pbClock = sysClock / 2;
			break;

		case pbDiv1:
			pbClock = sysClock / 1;
	}

	return pbClock;			
}
