/************************************************************************/
/*																		*/
/*	Dtimer.c	--	Interface Declarations for PIC32 Timers				*/
/*																		*/
/************************************************************************/
/*	Author:		Oliver Jones											*/
/*	Copyright 2011, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*																		*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	07/08/2011(OliverJ): created										*/
/*																		*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include <p32xxxx.h>
#include "stdTypes.h"
#include "Dport.h"

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Forward Declarations							*/
/* ------------------------------------------------------------ */


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */
/***	BOOL DportGetHandle(portHIF * hif, char chPort)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		chPort			- Port requested
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Initializes handle to a port determined by chPort
**
*/


BOOL DportGetHandle(portHIF * hif, char chPort)
{
	switch(chPort) {
	case 'A':
		#if defined (_PORTA)
			hif->pTrs = (unsigned int *)&TRISA;
			hif->pTrsSet = (unsigned int *)&TRISASET;
			hif->pTrsClr = (unsigned int *)&TRISACLR;
			hif->pLat = (unsigned int *)&LATA;
			hif->pLatSet = (unsigned int *)&LATASET;
			hif->pLatClr = (unsigned int *)&LATACLR;
			hif->pPort = (unsigned int *)&PORTA;
			break;
		#else
			setLastError(1);
			return fFalse;
		#endif

	case 'B':
		#if defined (_PORTB)
			hif->pTrs = (unsigned int *)&TRISB;
			hif->pTrsSet = (unsigned int *)&TRISBSET;
			hif->pTrsClr = (unsigned int *)&TRISBCLR;
			hif->pLat = (unsigned int *)&LATB;
			hif->pLatSet = (unsigned int *)&LATBSET;
			hif->pLatClr = (unsigned int *)&LATBCLR;
			hif->pPort = (unsigned int *)&PORTB;
			break;
		#else
			setLastError(1);
			return fFalse;
		#endif

	case 'C':
		#if defined (_PORTC)
			hif->pTrs = (unsigned int *)&TRISC;
			hif->pTrsSet = (unsigned int *)&TRISCSET;
			hif->pTrsClr = (unsigned int *)&TRISCCLR;
			hif->pLat = (unsigned int *)&LATC;
			hif->pLatSet = (unsigned int *)&LATCSET;
			hif->pLatClr = (unsigned int *)&LATCCLR;
			hif->pPort = (unsigned int *)&PORTC;
			break;
		#else
			setLastError(1);
			return fFalse;
		#endif

	case 'D':
		#if defined (_PORTD)
			hif->pTrs = (unsigned int *)&TRISD;
			hif->pTrsSet = (unsigned int *)&TRISDSET;
			hif->pTrsClr = (unsigned int *)&TRISDCLR;
			hif->pLat = (unsigned int *)&LATD;
			hif->pLatSet = (unsigned int *)&LATDSET;
			hif->pLatClr = (unsigned int *)&LATDCLR;
			hif->pPort = (unsigned int *)&PORTD;
			break;
		#else
			setLastError(1);
			return fFalse;
		#endif

	case 'E':
		#if defined (_PORTE)
			hif->pTrs = (unsigned int *)&TRISE;
			hif->pTrsSet = (unsigned int *)&TRISESET;
			hif->pTrsClr = (unsigned int *)&TRISECLR;
			hif->pLat = (unsigned int *)&LATE;
			hif->pLatSet = (unsigned int *)&LATESET;
			hif->pLatClr = (unsigned int *)&LATECLR;
			hif->pPort = (unsigned int *)&PORTE;
			break;
		#else
			setLastError(1);
			return fFalse;
		#endif

	case 'F':
		#if defined (_PORTF)
			hif->pTrs = (unsigned int *)&TRISF;
			hif->pTrsSet = (unsigned int *)&TRISFSET;
			hif->pTrsClr = (unsigned int *)&TRISFCLR;
			hif->pLat = (unsigned int *)&LATF;
			hif->pLatSet = (unsigned int *)&LATFSET;
			hif->pLatClr = (unsigned int *)&LATFCLR;
			hif->pPort = (unsigned int *)&PORTF;
			break;
		#else
			setLastError(1);
			return fFalse;
		#endif

	case 'G':
		#if defined (_PORTG)
			hif->pTrs = (unsigned int *)&TRISG;
			hif->pTrsSet = (unsigned int *)&TRISGSET;
			hif->pTrsClr = (unsigned int *)&TRISGCLR;
			hif->pLat = (unsigned int *)&LATG;
			hif->pLatSet = (unsigned int *)&LATGSET;
			hif->pLatClr = (unsigned int *)&LATGCLR;
			hif->pPort = (unsigned int *)&PORTG;
			break;
		#else
			setLastError(1);
			return fFalse;
		#endif

	default:
		setLastError(2);
		return fFalse;
	}	

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DportSetPortDir(portHIF * hif, unsigned int value)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		value			- Value to set tristate register
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Sets tristate register using the value given
**
*/

BOOL DportSetPortDir(portHIF * hif, unsigned int value)
{
	*(hif->pTrs) = value;

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DportSetPortDir(portHIF * hif, unsigned int value)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		pin				- pin to change direction
**		fSel			- determines whether pin is an input or 
**						  output
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Sets tristate register of a specified pin
**
*/

BOOL DportSetPinDir(portHIF * hif, unsigned int pin, BOOL fSel)
{
	if(pin >= 32) {
		return fFalse;
	}

	if(fSel == fTrue) {
		//Set as output		
		*(hif->pTrsClr) = (1 << pin);
	}
	else
	{
		//Set as input
		*(hif->pTrsSet) = (1 << pin);
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DportSetPortOutput(portHIF * hif, unsigned int value)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		value			- value to change port output
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Changes port output to spefied value
**
*/

BOOL DportSetPortOutput(portHIF * hif, unsigned int value)
{
	*(hif->pLat) = value;

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DportSetPinOutput(portHIF * hif, unsigned int pin, BOOL fSel)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		pin				- Pin
**		fSel			- Desired value of pin
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Changes pin output to spefied value determined by fSel
**		fTrue sets pin to logic high and fFalse sets pin to
**		logic low
**
*/

BOOL DportSetPinOutput(portHIF * hif, unsigned int pin, BOOL fSel)
{
	if(pin >= 32) {
		setLastError(2);
		return fFalse;
	}

	if(fSel == fTrue) {
		//Set pin to logic high
		*(hif->pLatSet) = (1 << pin);
	}
	else
	{
		*(hif->pLatClr) = (1 << pin);
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***	void DportInvertPortOutput(portHIF * hif)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Inverts all pins on the given port
**
*/

void DportInvertPortOutput(portHIF * hif)
{
	*hif->pLat= ~(*hif->pLat);
}

/* ------------------------------------------------------------ */
/***	void DportInvertPinOutput(portHIF * hif, unsigned int pin)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		pin				- Pin
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Inverts the specified pin
**
*/

void DportInvertPinOutput(portHIF * hif, unsigned int pin)
{
	//Need to Fix	
}

/* ------------------------------------------------------------ */
/***	BOOL DportGetPort(portHIF * hif, unsigned int *pRcv)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		pRcv			- Pointer to u int to return port value
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Reads port and returns to u int pointed by pbRcv
**
*/

BOOL DportGetPort(portHIF * hif, unsigned int *pRcv)
{
	*pRcv = *(hif->pPort);
	
	return fTrue;
}

/* ------------------------------------------------------------ */
/***	BOOL DportGetPort(portHIF * hif, unsigned int *pRcv)
**
**	Parameters:
**		hif				- Pointer to I/O port handle
**		pRcv			- Pointer to bool to return pin value
**
**	Return Value:
**		Returns True if succesful and false if not
**
**	Errors:
**		TBD
**
**	Description:
**		Reads pin and returns to u int pointed by pRcv
**
*/

BOOL DportGetPin(portHIF * hif, unsigned int pin, BOOL *pRcv)
{
	if(pin >= 32) {
		return fFalse;
	}	

	if((1 << pin) & *(hif->pPort)) {
		*pRcv = fTrue;
	}
	else {
		*pRcv = fFalse;
	}

	return fTrue;
}
